@@ -2,7 +2,8 @@
 # coding=utf-8
 import sys, os, traceback
 
-from pythontk.Iter import makeList, formatReturn, filterList
+from pythontk.Core import listify
+from pythontk.Iter import makeList, filterList
 
 
 class File():
@@ -12,7 +13,7 @@ class File():
 	def getFile(filepath, mode='a+'):
 		'''Return a file object with the given mode.
 
-		:Parameters:
+		Parameters:
 			filepath (str): The path to an existing file or the desired location for one to be created.
 			mode (str): 'r' - Read - Default value. Opens a file for reading, error if the file does not exist.
 				'a' - Append - Opens a file for appending, creates the file if it does not exist.
@@ -23,7 +24,7 @@ class File():
 				't' - Text - Default value. Text mode
 				'b' - Binary - Binary mode (e.g. images)
 
-		:Return:
+		Return:
 			(obj) file
 		'''
 		try:
@@ -34,19 +35,20 @@ class File():
 
 
 	@staticmethod
-	def getFileContents(filepath: str) -> None:
+	def getFileContents(filepath: str, asList=False) -> None:
 		'''Get each line of a text file as indices of a list.
 		Will create a file if one doesn't already exist.
 
-		:Parameters:
+		Parameters:
 			filepath (str): The path to an existing text based file.
+			asList (bool): Return as a list or a string.
 
-		:Return:
+		Return:
 			(list)
 		'''
 		try:
 			with open(filepath, 'r') as f:
-				return f.readlines()
+				return f.readlines() if asList else f.read()
 		except OSError as error:
 			traceback.print_exc()
 
@@ -55,7 +57,7 @@ class File():
 	def writeToFile(filepath, lines):
 		'''Write the given list contents to the given file.
 
-		:Parameters:
+		Parameters:
 			filepath (str): The path to an existing text based file.
 			lines (list): A list of strings to write to the file.
 		'''
@@ -67,13 +69,14 @@ class File():
 
 
 	@staticmethod
-	def formatPath(strings, section='', replace=''):
+	@listify
+	def formatPath(p, section='', replace=''):
 		'''Format a given filepath(s).
 		When a section arg is given, the correlating section of the string will be returned.
 		If a replace arg is given, the stated section will be replaced by the given value.
 
-		:Parameters:
-			strings (str)(list): The filepath(s) to be formatted.
+		Parameters:
+			p (str)(list): The filepath(s) to be formatted.
 			section (str): The desired subsection of the given path. 
 					'path' path - filename, 
 					'dir'  directory name, 
@@ -81,60 +84,57 @@ class File():
 					'name', filename - ext,
 					'ext', file extension,
 					(if '' is given, the fullpath will be returned)
-		:Return:
+		Return:
 			(str)(list) List if 'strings' given as list.
 		'''
-		result=[]
-		for s in makeList(strings):
-			if not isinstance(s, (str)):
-				continue
-			s = os.path.expandvars(s) #convert any env variables to their values.
-			s = s[:2]+'/'.join(s[2:].split('\\')).rstrip('/') #convert forward slashes to back slashes.
+		if not isinstance(p, (str)):
+			return p
 
-			fullpath = s if '/' in s else ''
-			filename_ = s.split('/')[-1]
-			filename = filename_ if '.' in filename_ and not filename_.startswith('.') else ''
-			path = '/'.join(s.split('/')[:-1]) if filename else s
-			directory = s.split('/')[-2] if (filename and path) else s.split('/')[-1]
-			name = ''.join(filename.rsplit('.', 1)[:-1]) if filename else '' if fullpath else s
-			ext = filename.rsplit('.', 1)[-1]
+		p = os.path.expandvars(p) #convert any env variables to their values.
+		p = p[:2]+'/'.join(p[2:].split('\\')).rstrip('/') #convert forward slashes to back slashes.
 
-			orig_str = s #the full original string (formatted with forwardslashes)
+		fullpath = p if '/' in p else ''
+		filename_ = p.split('/')[-1]
+		filename = filename_ if '.' in filename_ and not filename_.startswith('.') else ''
+		path = '/'.join(p.split('/')[:-1]) if filename else p
+		directory = p.split('/')[-2] if (filename and path) else p.split('/')[-1]
+		name = ''.join(filename.rsplit('.', 1)[:-1]) if filename else '' if fullpath else p
+		ext = filename.rsplit('.', 1)[-1]
 
-			if section=='path':
-				s = path
+		orig_str = p #the full original string (formatted with forwardslashes)
 
-			elif section=='dir':
-				s = directory
+		if section=='path':
+			p = path
 
-			elif section=='file':
-				s = filename
+		elif section=='dir':
+			p = directory
 
-			elif section=='name':
-				s = name
+		elif section=='file':
+			p = filename
 
-			elif section=='ext':
-				s = ext
+		elif section=='name':
+			p = name
 
-			if replace:
-				s = Str_utils.rreplace(orig_str, s, replace, 1)
+		elif section=='ext':
+			p = ext
 
-			result.append(s)
+		if replace:
+			p = Str_utils.rreplace(orig_str, p, replace, 1)
 
-		return formatReturn(result, strings) #if 'strings' is given as a list; return a list.
+		return p
 
 
 	@classmethod
 	def timeStamp(cls, filepaths, detach=False, stamp='%m-%d-%Y  %H:%M', sort=False):
 		'''Attach a modified timestamp and date to given file path(s).
 
-		:Parameters:
+		Parameters:
 			filepaths (str)(list): The full path to a file. ie. 'C:/Windows/Temp/__AUTO-SAVE__untitled.0001.mb'
 			detach (bool): Remove a previously attached time stamp.
 			stamp (str): The time stamp format.
 			sort (bool): Reorder the list of filepaths by time. (most recent first)
 
-		:Return:
+		Return:
 			(list) ie. ['16:46  11-09-2021  C:/Windows/Temp/__AUTO-SAVE__untitled.0001.mb'] from ['C:/Windows/Temp/__AUTO-SAVE__untitled.0001.mb']
 		'''
 		from datetime import datetime
@@ -167,10 +167,10 @@ class File():
 	def isValidPath(filepath: str) -> list:
 		'''Determine if the given filepath is valid.
 
-		:Parameters:
+		Parameters:
 			filepath (str): The path to a file.
 
-		:Return:
+		Return:
 			(str) The path type (ie. 'file' or 'dir') or None.
 		'''
 		fp = os.path.expandvars(filepath) #convert any env variables to their values.
@@ -186,7 +186,7 @@ class File():
 	def createDir(filepath: str) -> None:
 		'''Create a directory if one doesn't already exist.
 
-		:Parameters:
+		Parameters:
 			filepath (str): The path to where the file will be created.
 		'''
 		fp = os.path.expandvars(filepath) #convert any env variables to their values.
@@ -202,7 +202,7 @@ class File():
 								incFiles=[], excFiles=[], incDirs=[], excDirs=[]):
 		'''Get the contents of a directory and any of it's children.
 
-		:Parameters:
+		Parameters:
 			path (str): The path to the directory.
 			returnType (str): Return files and directories. Multiple types can be given using '|' 
 					ex. 'files|dirs' (valid: 'files'(default), filenames, 'filepaths', 'dirs', 'dirpaths')
@@ -217,7 +217,7 @@ class File():
 					supports using the '*' operator: startswith*, *endswith, *contains*
 					ex. *.ext will exclude all files with the given extension.
 					exclude takes precedence over include.
-		:Return:
+		Return:
 			(list)
 
 		ex. getDirContents(path, returnType='filepaths')
@@ -257,11 +257,11 @@ class File():
 	def getFilepath(obj, incFilename=False):
 		'''Get the filepath of a class or module.
 
-		:Parameters:
+		Parameters:
 			obj (obj): A python module, class, or the built-in __file__ variable.
 			incFilename (bool): Include the filename in the returned result.
 
-		:Return:
+		Return:
 			(str)
 		'''
 		from types import ModuleType
@@ -304,43 +304,75 @@ class File():
 
 
 	@classmethod
-	def incVersion(cls, filepath: str) -> None:
-		"""This function increments the version number in a text file.
-		The verson number is defined as a line in the following format: __version__ = "0.0.0"
+	def updateVersion(cls, filepath: str, change: str='increment', version_part: str='patch', max_version_parts: tuple=(9, 9)) -> None:
+		"""This function updates the version number in a text file depending on its state.
+		The version number is defined as a line in the following format: __version__ = "0.0.0"
 
 		The version number is represented as a string in the format 'x.y.z', where x, y, and z are integers. 
-		When the value of z reaches 9, it becomes 0 and the value of y is incremented by 1. 
-		When the value of y reaches 10, it becomes 0 and the value of x is incremented by 1. 
-		
-		:Parameters:
-			filepath (str): The path to the text file containing the version number.
 
-		:Return:
-			(None) The version number in the file is updated.
+		Parameters:
+			filepath (str): The path to the text file containing the version number.
+			change (str, optional): The type of change, either 'increment' or 'decrement'. Defaults to 'increment'.
+			version_part (str, optional): The part of the version number to update, either 'major', 'minor', or 'patch'. Defaults to 'patch'.
+			max_version_parts (tuple, optional): A tuple containing the maximum values for the minor and patch version parts. Defaults to (9, 9).
+
+		Return:
+			(str): The new version number.
 		"""
 		import re
 
-		lines = cls.getFileContents(filepath)
+		lines = cls.getFileContents(filepath, asList=True)
 
+		version_pattern = re.compile(r"__version__\s*=\s*['\"](\d+)\.(\d+)\.(\d+)['\"]")
+		max_minor, max_patch = max_version_parts
+
+		version = ''
 		for i, line in enumerate(lines):
-			if line.startswith("__version__"):
-				_, version = line.strip().split(" = ")
-				version = version.strip("'")
-				major, minor, patch = map(int, version.split("."))
-				if patch == 9:
-					if minor == 9:
-						minor = 0
-						major += 1
+			match = version_pattern.match(line)
+			if match:
+				major, minor, patch = map(int, match.groups())
+
+				if version_part == 'patch':
+					if change == 'increment':
+						patch = (patch + 1) % (max_patch + 1)
+						if patch == 0:
+							minor = (minor + 1) % (max_minor + 1)
+							major += minor == 0
+					elif change == 'decrement':
+						if patch == 0:
+							patch = max_patch
+							minor = (minor - 1) % (max_minor + 1) if minor > 0 else max_minor
+							major -= minor == max_minor
+						else:
+							patch -= 1
 					else:
-						minor += 1
-					patch = 0
+						raise ValueError("Invalid change parameter. Use either 'increment' or 'decrement'.")
+				elif version_part == 'minor':
+					if change == 'increment':
+						minor = (minor + 1) % (max_minor + 1)
+						major += minor == 0
+					elif change == 'decrement':
+						minor = (minor - 1) % (max_minor + 1)
+					else:
+						raise ValueError("Invalid change parameter. Use either 'increment' or 'decrement'.")
+				elif version_part == 'major':
+					if change == 'increment':
+						major += 1
+					elif change == 'decrement':
+						major = max(0, major - 1)
+					else:
+						raise ValueError("Invalid change parameter. Use either 'increment' or 'decrement'.")
 				else:
-					patch += 1
+					raise ValueError("Invalid version_part parameter. Use either 'major', 'minor', or 'patch'.")
+
 				version = f"{major}.{minor}.{patch}"
 				lines[i] = f"__version__ = '{version}'\n"
 				break
 
 		cls.writeToFile(filepath, lines)
+		if not version:
+			print (f"# Error: No version in the format: __version__ = \"0.0.0\" found in {filepath}")
+		return version
 
 # --------------------------------------------------------------------------------------------
 
@@ -357,22 +389,20 @@ class File():
 def __getattr__(attr:str):
 	"""Searches for an attribute in this module's classes and returns it.
 
-	:Parameters:
+	Parameters:
 		attr (str): The name of the attribute to search for.
 	
-	:Return:
+	Return:
 		(obj) The found attribute.
 
 	:Raises:
 		AttributeError: If the given attribute is not found in any of the classes in the module.
 	"""
-	import sys
-	from pythontk import searchClassesForAttr
+	try:
+		return getattr(File, attr)
 
-	attr = searchClassesForAttr(sys.modules[__name__], attr)
-	if not attr:
+	except AttributeError as error:
 		raise AttributeError(f"Module '{__name__}' has no attribute '{attr}'")
-	return attr
 
 # --------------------------------------------------------------------------------------------
 
@@ -385,4 +415,20 @@ if __name__=='__main__':
 
 
 
-# Deprecated ------------------------------------
\ No newline at end of file
+# Deprecated ------------------------------------
+
+
+	# def getCallingModuleDir():
+	# 	"""Get the directory path of the module that called the function.
+
+	# 	Return:
+	# 		(str) The directory path of the calling module.
+	# 	"""
+	# 	import os, inspect
+
+	# 	calling_frame = inspect.currentframe().f_back
+	# 	calling_module = inspect.getmodule(calling_frame)
+	# 	calling_module_path = os.path.abspath(calling_module.__file__)
+	# 	calling_module_dir = os.path.dirname(calling_module_path)
+
+	# 	return calling_module_dir
\ No newline at end of file
