@@ -1,75 +1,115 @@
 # !/usr/bin/python
 # coding=utf-8
-from pythontk.Iter import makeList, formatReturn
+from pythontk.Core import listify
+from pythontk.Iter import makeList
 
 
 class Str():
 	'''
 	'''
 	@staticmethod
-	def setCase(strings, case='camel'):
+	@listify
+	def setCase(string, case='camel'):
 		'''Format the given string(s) in the given case.
-		
-		:Parameters:
-			strings (str)(list): The string(s) to format.
+
+		Parameters:
+			string (str)(list): The string(s) to format.
 			case (str): The desired return case. Accepts all python case operators. 
 				valid: 'upper', 'lower', 'capitalize' (default), 'swapcase', 'title', 'pascal', 'camel', None.
 
-		:Return:
+		Return:
 			(str)(list) List if 'string' given as list.
 		'''
-		if not strings or not isinstance(strings, str):
-			return strings
+		if (not string) or (not isinstance(string, str)):
+			return ''
 
 		if case=='pascal':
-			result = [s[:1].capitalize()+s[1:] for s in makeList(strings)] #capitalize the first letter.
+			return string[:1].capitalize()+string[1:] #capitalize the first letter.
 
 		elif case=='camel':
-			result = [s[0].lower()+s[1:] for s in makeList(strings)] #lowercase the first letter.
+			return string[0].lower()+string[1:] #lowercase the first letter.
 
 		else:
 			try:
-				result = [getattr(s, case)() for s in makeList(strings)]
+				return getattr(string, case)()
 
-			except AttributeError as error: #return the original string(s).
-				return strings
-
-		return formatReturn(result, strings) #if 'strings' is given as a list; return a list.
+			except AttributeError: #return the original string.
+				return string
 
 
 	@staticmethod
-	def splitAtChars(strings, chars='|', occurrence=-1):
+	def getTextBetweenDelimiters(string, start_delim, end_delim, as_string=False):
+		"""Get any text between the specified start and end delimiters in the given string. The text can be returned as a
+		generator (default behavior) or as a single concatenated string if `as_string` is set to True.
+
+		Parameters:
+			string (str): The input string to search for matches.
+			start_delim (str): The starting delimiter to search for.
+			end_delim (str): The ending delimiter to search for.
+			as_string (bool, optional): If True, the function returns a single concatenated string of all matches.
+										 If False (default), the function returns a generator that yields each match.
+
+		Returns:
+			If as_string is False (default): A generator that yields all matches found in the input string.
+			If as_string is True: A single concatenated string containing all matches found in the input string.
+
+		Examples:
+			input_string = "Here is the <!-- start -->first match<!-- end --> and here is the <!-- start -->second match<!-- end -->"
+
+			# Get the matches as a generator (default behavior)
+			matches_generator = getTextBetweenDelimiters(input_string, '<!-- start -->', '<!-- end -->')
+			for match in matches_generator:
+				print(match)  # Output: first match (first iteration), second match (second iteration)
+
+			# Get the matches as a single string
+			matches_string = getTextBetweenDelimiters(input_string, '<!-- start -->', '<!-- end -->', as_string=True)
+			print(matches_string)  # Output: "first match second match"
+		"""
+		import re
+
+		def extract_matches(string, start_delim, end_delim, start_index=0):
+			pattern = re.compile(f'{re.escape(start_delim)}(.*?){re.escape(end_delim)}', re.DOTALL)
+			match = pattern.search(string, start_index)
+			if match:
+				yield match.group(1).strip()
+				yield from extract_matches(string, start_delim, end_delim, match.end())
+
+		if as_string:
+			matches = list(extract_matches(string, start_delim, end_delim))
+			return " ".join(matches)
+		else:
+			return extract_matches(string, start_delim, end_delim)
+
+
+	@staticmethod
+	@listify
+	def splitAtChars(string, chars='|', occurrence=-1):
 		'''Split a string containing the given chars at the given occurrence and return
 		a two element tuple containing both halves.
 
-		:Parameters:
+		Parameters:
 			strings (str)(list): The string(s) to operate on.
 			chars (str): The chars to split at.
 			occurrence (int): The occurrence of the pipe to split at from left.
 				ex. -1 would split at the last occurrence. 0 would split at the first.
 					If the occurrence is out of range, the full string will be 
 					returned as: ('original string', '')
-		:Return:
+		Return:
 			(tuple)(list) two element tuple, or list of two element tuples if multiple strings given.
 
-		:Example: splitAtChars(['str|ing', 'string']) returns: [('str', 'ing'), ('string', '')]
+		Example: splitAtChars(['str|ing', 'string']) returns: [('str', 'ing'), ('string', '')]
 		'''
-		result = []
-		for s in makeList(strings):
-			split = s.split(chars)
+		split = string.split(chars)
 
-			try:
-				s2 = ''.join(split[occurrence])
-				if chars in s:
-					s1 = chars.join(split[:occurrence])
-				else:
-					s1, s2 = (s2, '')
-			except IndexError as error:
-				s1, s2 = (s, '')
-
-			result.append((s1, s2))
-
-		return formatReturn(result, strings) #if 'strings' is given as a list; return a list.
+		try:
+			s2 = ''.join(split[occurrence])
+			if chars in string:
+				s1 = chars.join(split[:occurrence])
+				return (s1, s2)
+			else:
+				return (s2, '')
+		except IndexError as error:
+			return (string, '')
 
 
 	@classmethod
@@ -77,7 +117,7 @@ class Str():
 		'''Insert character(s) into a string at a given location.
 		if the character doesn't exist, the original string will be returned.
 
-		:Parameters:
+		Parameters:
 			src (str): The source string.
 			ins (str): The character(s) to insert.
 			at (str)(int): The index or char(s) to insert at.
@@ -87,7 +127,7 @@ class Str():
 							(A value of -1 would insert at the last occurrence)
 			before (bool): Specify inserting before or after. default: after
 							Valid only when 'at' is given as a string.
-		:Return:
+		Return:
 			(str)
 		'''
 		try:
@@ -106,13 +146,13 @@ class Str():
 		'''Replace occurrances in a string from right to left.
 		The number of occurrances replaced can be limited by using the 'count' argument.
 
-		:Parameters:
+		Parameters:
 			string (str): 
 			old (str): 
 			new (str)(int): 
 			count (int): 	
 
-		:Return:
+		Return:
 			(str)
 		'''
 		if not string or not isinstance(string, str):
@@ -125,19 +165,20 @@ class Str():
 
 
 	@staticmethod
+	@listify
 	def truncate(string, length=75, beginning=True, insert='..'):
 		'''Shorten the given string to the given length.
 		An ellipsis will be added to the section trimmed.
 
-		:Parameters:
+		Parameters:
 			length (int): The maximum allowed length before trunicating.
 			beginning (bool): Trim starting chars, else; ending.
 			insert (str): Chars to add at the trimmed area. (default: ellipsis)
 
-		:Return:
+		Return:
 			(str)
 
-		:Example: truncate('12345678', 4) #returns: '..5678'
+		Example: truncate('12345678', 4) #returns: '..5678'
 		'''
 		if not string or not isinstance(string, str):
 			return string
@@ -154,7 +195,7 @@ class Str():
 	def getTrailingIntegers(string, inc=0, asString=False):
 		'''Returns any integers from the end of the given string.
 
-		:Parameters:
+		Parameters:
 			inc (int): Increment by a step amount. (default: 0)
 					0 does not increment and returns the original number.
 			asString (bool): Return the integers as a string instead of integers.
@@ -181,7 +222,7 @@ class Str():
 	def findStr(find, strings, regEx=False, ignoreCase=False):
 		'''Filter for elements that containing the given string in a list of strings.
 
-		:Parameters:
+		Parameters:
 			find (str): The search string. An asterisk denotes startswith*, *endswith, *contains*, and multiple search strings can be separated by pipe chars.
 				wildcards:
 					*chars* - string contains chars.
@@ -199,7 +240,7 @@ class Str():
 			regEx (bool): Use regular expressions instead of wildcards.
 			ignoreCase (bool): Search case insensitive.
 
-		:Return:
+		Return:
 			(list)
 
 		ex. lst = ['invertVertexWeights', 'keepCreaseEdgeWeight', 'keepBorder', 'keepBorderWeight', 'keepColorBorder', 'keepColorBorderWeight']
@@ -255,7 +296,7 @@ class Str():
 	def findStrAndFormat(cls, strings, to, fltr='', regEx=False, ignoreCase=False, returnOldNames=False):
 		'''Expanding on the 'findStr' function: Find matches of a string in a list of strings and re-format them.
 
-		:Parameters:
+		Parameters:
 			strings (list): A list of string objects to search.
 			to (str): An optional asterisk modifier can be used for formatting. An empty string will attempt to remove the part of the string designated in the from argument.
 				"" - (empty string) - strip chars.
@@ -269,7 +310,7 @@ class Str():
 			ignoreCase (bool): Ignore case when searching. Applies only to the 'fltr' parameter's search.
 			returnOldNames (bool): Return the old names as well as the new.
 
-		:Return:
+		Return:
 			(list) if returnOldNames: list of two element tuples containing the original and modified string pairs. [('frm','to')]
 				else: a list of just the new names.
 		'''
@@ -344,14 +385,14 @@ class Str():
 	def formatSuffix(string, suffix='', strip='', stripTrailingInts=False, stripTrailingAlpha=False):
 		'''Re-format the suffix for the given string.
 
-		:Parameters:
+		Parameters:
 			string (str): The string to format.
 			suffix (str): Append a new suffix to the given string.
 			strip (str)(list): Specific string(s) to strip from the end of the given string.
 			stripTrailingInts (bool): Strip all trailing integers.
 			stripTrailingAlpha (bool): Strip all upper-case letters preceeded by a non alphanumeric character.
 
-		:Return:
+		Return:
 			(str)
 		'''
 		import re
@@ -393,22 +434,20 @@ class Str():
 def __getattr__(attr:str):
 	"""Searches for an attribute in this module's classes and returns it.
 
-	:Parameters:
+	Parameters:
 		attr (str): The name of the attribute to search for.
 	
-	:Return:
+	Return:
 		(obj) The found attribute.
 
 	:Raises:
 		AttributeError: If the given attribute is not found in any of the classes in the module.
 	"""
-	import sys
-	from pythontk import searchClassesForAttr
+	try:
+		return getattr(Str, attr)
 
-	attr = searchClassesForAttr(sys.modules[__name__], attr)
-	if not attr:
+	except AttributeError as error:
 		raise AttributeError(f"Module '{__name__}' has no attribute '{attr}'")
-	return attr
 
 # --------------------------------------------------------------------------------------------
 
